/****************************************************************************
 *            telnet.c
 *
 *  Mon Jun 19 21:55:59 2006
 *  Copyright  2006  sharandac
 *  Email sharandac(at)snafu.de
 ****************************************************************************/
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>

#include "../hardware/timer1.h"
#include "../hardware/mem-check.h"
#include "../hardware/enc28j60.h"

#include "../net/network.h"
#include "telnet.h"

unsigned int SOCKETSTATE = NO_SOCKET_USED;

void telnet_init()
	{
		RegisterTCPPort( TELNET_PORT );
		printf_P( PSTR("Telnet-Server gestartet auf Port %d.\r\n") , TELNET_PORT );
	}
	
void telnet_thread()
	{
		unsigned char * buffer;
		buffer = (unsigned char*) __builtin_alloca (( size_t ) TELNET_BUFFER );
		
		unsigned int datalen,i;
		
		union IP_ADDRESS DNSIP;
		
		// checken ob die Verbindung abgerissen ist oder ein Timeout statt fand
		if ( SOCKETSTATE != NO_SOCKET_USED ) 
			if( CheckSocketState( SOCKETSTATE ) == SOCKET_NOT_USE )
			{
				CloseTCPSocket( SOCKETSTATE );
				SOCKETSTATE = NO_SOCKET_USED;
				printf_P( PSTR("Verbindung abgerissen oder timeout\r\n"));
				return;
			}

		if ( SOCKETSTATE == NO_SOCKET_USED )
		  { // keine Verbinfung offen?
			SOCKETSTATE = CheckPortRequest( TELNET_PORT );
			if ( SOCKETSTATE != NO_SOCKET_USED )
			{
				static const char MSG[] PROGMEM = {
					"Welcome on Tiny-Telnetserver 0.1a!\r\n"
					"Und, Weiter Weiter!!!\r\n" }	;
				PutSocketData_P( SOCKETSTATE , strlen_P( MSG ) , MSG );
			}
		}
		else
		{
			datalen = GetSocketNextLine( SOCKETSTATE , TELNET_BUFFER , buffer );
		
			if ( datalen != 0)
			{
				// alles auf Großschreibung umbauen
				for ( i = 0  ; i < datalen ; i++ ) buffer[i] = toupper( buffer[i] );
				
				// auf STATS checken
				if ( !memcmp( &buffer[0] , "STATS" , 5 ) ) 
				{
					unsigned int REVID,_PHSTAT1,_PHSTAT2;

					// buffer mit ausgabe vorbereiten & ausgeben
					sprintf_P( buffer , PSTR("Ethernet: %ld Bytes in %ld Packeten\r\n") , ByteCounter, PacketCounter );
					PutSocketData( SOCKETSTATE , strlen( &buffer[0] ) , &buffer[0] );
					sprintf_P( buffer, PSTR("TCP-TX Errors: %d TCP-RX Errors: %d\r\n\r\n"), TXErrorCounter , RXErrorCounter);
					PutSocketData( SOCKETSTATE, strlen( buffer ), buffer );

					LockEthernet();
					REVID = enc28j60Read( EREVID );
					_PHSTAT1 = enc28j60PhyRead( PHSTAT1 );
					_PHSTAT2 = enc28j60PhyRead( PHSTAT2 );
					FreeEthernet();
					
					sprintf_P( buffer , PSTR("ENC28J60:\r\n"));
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					sprintf_P( buffer , PSTR("Reg.    | Wert\r\n"));
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					sprintf_P( buffer , PSTR("--------+--------\r\n"));
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					sprintf_P( buffer , PSTR("Rev.ID  |   0x%02X\r\n") , REVID );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					sprintf_P( buffer , PSTR("PHSTAT1 | 0x%04X\r\n") , _PHSTAT1 );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					sprintf_P( buffer , PSTR("PHSTAT2 | 0x%04X\r\n") , _PHSTAT2 );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );

				}

				// auf QUIT checken
				if ( !memcmp( &buffer[0] , "QUIT" , 4 ) ) 
				{
					// Socket schließen
					sprintf_P( buffer , PSTR("Verbindung wird geschlossen\r\n") );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
					CloseTCPSocket( SOCKETSTATE );
					SOCKETSTATE = NO_SOCKET_USED;
				}

				// auf OSCCAL checken
				if ( !memcmp( &buffer[0] , "OSCCAL" , 6 ) ) 
				{
					// OSCCAL-wert einlesen und setzen
					if ( atoi( &buffer[7] ) ) OSCCAL = atoi( &buffer[7] );
					// ausgaben von OSCCAL-Register vorbereiten & senden
					sprintf_P( buffer , PSTR("OSCCAL = %2d\r\n") ,OSCCAL );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
				}

				// auf TIME checken
				if ( !memcmp( &buffer[0] , "TIME" , 4 ) )
				{
					// Zeit einlesen & setzen wenn parameter angebenen wurden
					if ( atoi( &buffer[5] ) ) {
						hh = atoi( &buffer[5] );
						mm = atoi( &buffer[8] );
						ss = atoi( &buffer[11] );
					}
					// ausgaben vorbereiten und senden
					sprintf_P( buffer , PSTR("Time: %02d:%02d:%02d\r\n") , hh , mm , ss);
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
				}

				// auf MEM checken
				if ( !memcmp( &buffer[0] , "MEM" , 3 ) )
				{
					// ausgaben vorbereiten & senden
					sprintf_P( buffer , PSTR("Maximaler freier Speicher: %d\r\n") , get_mem_unused() );
					PutSocketData( SOCKETSTATE , strlen( buffer ) , buffer );
				}

				// auf HELP checken
				if ( !memcmp( &buffer[0] , "HELP" , 4 ) )
				{
					// Hilfetext senden aus dem Flash
					static const char helptxt[] PROGMEM = { 
							"\r\n Help\r\n======\r\n\r\n"
							"mem    = zeigt den maximalen freien speicher\r\n"
							"osccal = zeigt/setzt den internen osccal\r\n"
							"quit   = wat wohl ?\r\n"
							"stats  = statistiken\r\n"
							"time   = zeigt/setzt die uhr (time hh:mm:ss)\r\n\r\n" } ;
					PutSocketData_P( SOCKETSTATE , strlen_P( helptxt ) , helptxt );
				}
			PutSocketData_P( SOCKETSTATE , 2 , PSTR("> ") );
			}
		}
}
